@@ -26,6 +26,15 @@ class xdict(dict):
         
         self.update(d, **kwargs)
         
+    def __str__(self):
+        return dict.__str__(self.toDict()) if self else ''
+        
+    def __unicode__(self):
+        return dict.__unicode__(self.toDict()) if self else ''
+    
+    def __repr__(self):
+        return '%s(%s)' % (self.__class__.__name__, dict.__repr__(self))
+        
     def __getitem__(self, k):
         if isinstance(k, (str, unicode)) and '.' in k:
             k, rest = k.split('.', 1)
@@ -40,8 +49,13 @@ class xdict(dict):
     def __getattr__(self, k):
         try:
             return object.__getattribute__(self, k)
-        except AttributeError:
-            return self[k]
+        except AttributeError, e:
+            if k.startswith('__'):
+                raise e
+            try:
+                return self[k]
+            except KeyError:
+                raise e
         
     def get(self, k, default = None):
         v = self[k]
@@ -114,7 +128,45 @@ class xdict(dict):
                     raise AttributeError(k)
             else:
                 object.__delattr__(self, k)
+                
+    def toDict(self):
+        return to_dict(self)
+    
+    @staticmethod
+    def fromDict(d):
+        return xdict(d)
  
+def to_dict(x):
+    if isinstance(x, xdict):
+        return dict((k, to_dict(v)) for k, v in x.items())
+    elif isinstance(x, (tuple, list)):
+        return type(x)(to_dict(v) for v in x)
+    else:
+        return x
+    
+try:
+    try:
+        import json
+    except ImportError:
+        import simplejson as json
+        
+    class _JSONEncoder(json.encoder.JSONEncoder):
+        def default(self, o):
+            return str(o)
+                    
+    def toJSON(self, **kwargs):
+        return json.dumps(self, cls = _JSONEncoder, **kwargs)
+
+    def fromJSON(s, **kwargs):
+        return xdict(json.loads(s))
+    
+    xdict.toJSON = toJSON
+    xdict.fromJSON = staticmethod(fromJSON)
+    
+    
+except ImportError:
+    pass
+
 # class ValueBag(dict):
 #     def __init__(self, d = {}):
 #         super(ValueBag, self).__init__()
